@@ -77,10 +77,8 @@ def insert_one(item: dict, table_name: str, table_schema = None, skip_if_pk_matc
         statement = select(statement)
         item_exists = filehelper.sql(statement).fetchone()[0]
         if item_exists:
-            print("Item already exists")
             return
 
     statement = insert(table).values(**item)
-    print(statement)
 
     return filehelper.sql(statement)
