@@ -2,8 +2,10 @@ import io
 import os
 import re
 import requests
+import boto3
+import s3fs
+import sqlalchemy
 
-import jhdata.connections as conn
 from jhdata.logger import *
 from jhdata.data_tools.schemas import *
 from jhdata.secrets import SecretInterfaceMemory
@@ -20,16 +22,61 @@ class FileHelper:
         self.bucket_name = bucket_name if bucket_name else self.environment
         self.logger = logger if logger else Logger()
         self.engine = engine
+        self.fs = None
+        self.boto = None
         self.secrets = secrets if secrets is not None else SecretInterfaceMemory()
 
     def set_engine(self, engine):
         self.engine = engine
 
     def get_engine(self):
-        return self.engine if self.engine is not None else conn.db_engine
+        if isinstance(self.engine, sqlalchemy.engine.Engine):
+            return self.engine
+        else:
+            print(self.engine)
+            raise Exception("No database engine defined, use connect_db() first")
 
-    def sql(self, statement, **kwargs):
-        with self.engine.connect() as connection:
+    def connect_db(self, connection_string: str):
+        self.set_engine(sqlalchemy.create_engine(connection_string))
+        return self.get_engine()
+
+    def connect_s3(self,
+                   endpoint_url: str,
+                   aws_access_key_id: str,
+                   aws_secret_access_key: str,
+                   aws_session_token: str = None):
+        s3fs_config = {
+            "anon": False,
+            "key": aws_access_key_id,
+            "secret": aws_secret_access_key,
+            "token": aws_session_token,
+            "client_kwargs": {
+                "endpoint_url": endpoint_url,
+            },
+            "config_kwargs": {
+                "signature_version": "s3v4",
+            }
+        }
+
+        boto3_config = {
+            "service_name": "s3",
+            "endpoint_url": endpoint_url,
+            "aws_access_key_id": aws_access_key_id,
+            "aws_secret_access_key": aws_secret_access_key,
+            "aws_session_token": aws_session_token,
+            "verify": True
+        }
+
+        print("s3fs_config:", s3fs_config)
+        print("boto3_config:", boto3_config)
+
+        self.fs = s3fs.S3FileSystem(**s3fs_config)
+        self.boto = boto3.resource(**boto3_config)
+
+        return self.fs, self.boto
+
+    def sql(self, statement, **kwargs) -> sqlalchemy.CursorResult:
+        with self.get_engine().connect() as connection:
             with connection.begin():
                 return connection.execute(statement, **kwargs)
 
@@ -38,19 +85,19 @@ class FileHelper:
         return f"{self.bucket_name}/{path}"
 
     def metadata(self, path: str):
-        return conn.fs.metadata(self.fspath(path))
+        return self.fs.metadata(self.fspath(path))
 
     # Last modified as datetime
     def last_modified(self, path: str) -> datetime.datetime:
-        return conn.fs.modified(self.fspath(path))
+        return self.fs.modified(self.fspath(path))
 
     # Check if path is a directory
     def is_directory(self, path: str) -> bool:
-        return conn.fs.isdir(self.fspath(path))
+        return self.fs.isdir(self.fspath(path))
 
     # Check if a path exists
     def path_exists(self, path: str) -> bool:
-        return conn.fs.exists(self.fspath(path))
+        return self.fs.exists(self.fspath(path))
 
     # Check if a path exists and is a directory
     def directory_exists(self, path: str) -> bool:
@@ -61,9 +108,13 @@ class FileHelper:
         return self.path_exists(path) and not self.is_directory(path)
 
     def find(self, path: str, **kwargs):
-        results = conn.fs.find(path=self.fspath(path), **kwargs)
+        print(self.fspath(path))
+        results = self.fs.find(path=self.fspath(path), **kwargs)
         return ["/".join(result.split("/")[1:]) for result in results]
 
+    def ls(self, path: str, **kwargs):
+        return self.fs.ls(path=self.fspath(path), **kwargs)
+
     def find_regex(self, path: str, regex: str, **kwargs):
         files = self.find(path, **kwargs)
         results = []
@@ -78,40 +129,40 @@ class FileHelper:
 
     # Generate an URL to access a file
     def url(self, path: str, **kwargs):
-        return conn.fs.url(path=self.fspath(path), **kwargs)
+        return self.fs.url(path=self.fspath(path), **kwargs)
 
     # Move file
     def move(self, path_from: str, path_to: str, recursive: bool = False):
-        conn.fs.move(self.fspath(path_from), self.fspath(path_to), recursive=recursive)
+        self.fs.move(self.fspath(path_from), self.fspath(path_to), recursive=recursive)
 
     # Copy file
     def copy(self, path_from: str, path_to: str, recursive: bool = False):
-        conn.fs.copy(self.fspath(path_from), self.fspath(path_to), recursive=recursive)
+        self.fs.copy(self.fspath(path_from), self.fspath(path_to), recursive=recursive)
 
     # Delete file
     def delete(self, path: str, recursive: bool = False):
-        conn.fs.rm(self, self.fspath(path), recursive=recursive)
+        self.fs.rm(self, self.fspath(path), recursive=recursive)
 
     # Get a readable file object
     def get_readable(self, path: str):
-        return conn.fs.open(self.fspath(path))
+        return self.fs.open(self.fspath(path))
 
     # Get a readable file object
     def get_readable_bytes(self, path: str):
-        return conn.fs.open(self.fspath(path), "rb")
+        return self.fs.open(self.fspath(path), "rb")
 
     # Get a writeable file object
     def get_writeable(self, path: str):
-        return conn.fs.open(self.fspath(path), "w")
+        return self.fs.open(self.fspath(path), "w")
 
     # Get a writeable file object
     def get_writeable_bytes(self, path: str):
-        return conn.fs.open(self.fspath(path), "wb")
+        return self.fs.open(self.fspath(path), "wb")
 
     # Download a file into a ByteIO object
     def read_byteio(self, path: str):
         output = io.BytesIO()
-        conn.boto.Bucket(self.bucket_name).download_fileobj(Key=path, Fileobj=output)
+        self.boto.Bucket(self.bucket_name).download_fileobj(Key=path, Fileobj=output)
         return output
 
     # Download file content as bytes
@@ -122,7 +173,7 @@ class FileHelper:
     # Upload bytes into a file
     def write_bytes(self, path: str, data: bytes):
         self.logger.info(f"[{self.bucket_name}] Uploading bytes to {path}")
-        result = conn.boto.Bucket(self.bucket_name).put_object(Key=path, Body=data)
+        result = self.boto.Bucket(self.bucket_name).put_object(Key=path, Body=data)
         return result
 
     # Download a file from an URL to bucket using HTTP
@@ -139,7 +190,7 @@ class FileHelper:
     # Upload text into a file
     def write_text(self, path: str, text: str, encoding: str = "utf8"):
         self.logger.info(f"[{self.bucket_name}] Uploading text to {path}")
-        result = conn.boto.Bucket(self.bucket_name).put_object(Key=path, Body=text.encode(encoding))
+        result = self.boto.Bucket(self.bucket_name).put_object(Key=path, Body=text.encode(encoding))
         return result
 
     # Read a JSON file into a Pandas DataFrame
@@ -165,8 +216,8 @@ class FileHelper:
 
     # Read a SQL table into a DataFrame
     def read_sql_table(self, table_name: str, table_schema: Schema = None, **kwargs):
-        engine = self.get_engine()
-        return pd.read_sql_table(table_name, engine, **kwargs)
+        with self.get_engine().begin() as conn:
+            return pd.read_sql_table(table_name, conn, **kwargs)
 
     # Write a DataFrame into a SQL table
     def write_sql(self, table_name: str, df: pd.DataFrame, table_schema: Schema = None, if_exists="replace", index=False, **kwargs):
